#!/usr/bin/env bash

# Usage ./upgrade_cse.sh kubeconfig-absolute-file-path [image_registry]


extract() {
  # Usage extract [tkg|vmware-cloud-director] [container-name:version] [destination folder]
  mkdir -p /tmp/container-fs
  mkdir -p $3
  docker pull ${image_registry}/$1/$2
  docker create --name "airgapped_container" ${image_registry}/$1/$2
  docker export "airgapped_container" | tar -C /tmp/container-fs -xvf -
  cp /tmp/container-fs/src/artifact/* $3
  docker rm "airgapped_container"
  rm -rf /tmp/container-fs
}

remove_folder() {
  # Usage remove_folder [folder path]
  if [[ -n "$1" ]];
  then
    echo "Removing folder: $1"
    rm -rf $1
    echo "$1 removed"
  else
    echo "Folder path was not set, and therefore cannot be removed"
    exit 1
  fi
}

if [[ "$#" < 1 ]];
then
	echo "Usage: $0 kubeconfig-absolute-file-path [image_registry]"
	exit 1
fi

if [[ ! -f $1 ]];
then
  echo "Error: cluster kubeconfig file: $1 not found"
  exit 1
fi

if [[ ! -n "$(command -v docker)" ]]
then
  echo "Error: docker command does not exist, please install docker on the system by referring to the README.md"
  exit 1
fi

if [[ ! -n "$(command -v kubectl)" ]]
then
  echo "Error: kubectl command does not exist, please install kubectl on the system by referring to the README.md"
  exit 1
fi

if [[ ! -n "$HOME" ]]
then
  echo "Info: \$HOME environment variable is empty or does not exist. Setting \$HOME to $(echo ~)"
  export HOME=$(echo ~)
fi

set -e

# Set the versions of the target components
# NOTE: capvcd version requires prefix "v" before version, when creating folder structure, clustercrtl.yaml to pass clusterctl upgrade
# CAPVCD version v1.1.0 is built with cluster-api v1.4.0, so it is expected to update CAPI components to v1.4.0.
capi_version="v1.4.0"
kubeadm_bootstrap_version="v1.4.0"
kubeadm_control_plane_version="v1.4.0"
clusterctl_version="v1.4.0"
capvcd_version="v1.1.0"
cert_manager_version="v1.11.1"
image_registry=${2:-projects.registry.vmware.com}
base_project="vmware-cloud-director"

rde_projector_image_name="vcd-rde-projector"
rde_projector_version="0.6.0"
cpi_image_name="cloud-provider-for-cloud-director"
cpi_version="1.4.0"
csi_image_name="cloud-director-named-disk-csi-driver"
csi_version="1.4.0"


# Create folder structure
CLUSTER_PACKAGES_FOLDER="${HOME}/cluster-upgrade-packages"

mkdir -p "${HOME}/.cluster-api"

# There is no need to create ${CLUSTER_PACKAGES_FOLDER} folder on it's own, as the next few lines will create that structure as -p flag works for the entire folder path.
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/cert-manager/${cert_manager_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}


#clusterctl install
extract vmware-cloud-director clusterctl-airgapped:${clusterctl_version} ${CLUSTER_PACKAGES_FOLDER}/clusterctl_artifact
chmod +x ${CLUSTER_PACKAGES_FOLDER}/clusterctl_artifact/clusterctl

#capvcd manifest
extract vmware-cloud-director capvcd-manifest-airgapped:${capvcd_version} ${CLUSTER_PACKAGES_FOLDER}/capvcd_artifact
cp ${CLUSTER_PACKAGES_FOLDER}/capvcd_artifact/infrastructure-components.yaml ${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capvcd_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}

#cluster-api components
extract vmware-cloud-director capi-airgapped:${capi_version} ${CLUSTER_PACKAGES_FOLDER}/capi_artifact

#copy bootstrap provider components
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/bootstrap-components.yaml ${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}

#copy core-capi components
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/core-components.yaml ${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}

#copy kubeadm control plane components
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/control-plane-components.yaml ${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}

#extract cert manager
extract vmware-cloud-director cert-manager-airgapped:${cert_manager_version} ${CLUSTER_PACKAGES_FOLDER}/cert_manager_artifact
cp ${CLUSTER_PACKAGES_FOLDER}/cert_manager_artifact/cert-manager.yaml ${CLUSTER_PACKAGES_FOLDER}/cert-manager/${cert_manager_version}

#extract csi manifest
extract vmware-cloud-director csi-crs-airgapped:${csi_version} ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped

#extract cpi manifest
extract vmware-cloud-director cpi-crs-airgapped:${cpi_version} ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped

# Generate clusterctl.yaml
cat << EOF > ${HOME}/.cluster-api/clusterctl.yaml
providers:
  - name: "vcd"
    url: "${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}/infrastructure-components.yaml"
    type: "InfrastructureProvider"
  - name: "cluster-api"
    url: "${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}/core-components.yaml"
    type: "CoreProvider"
  - name: "kubeadm"
    url: "${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}/bootstrap-components.yaml"
    type: "BootstrapProvider"
  - name: "kubeadm"
    url: "${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}/control-plane-components.yaml"
    type: "ControlPlaneProvider"
cert-manager:
  url: "${CLUSTER_PACKAGES_FOLDER}/cert-manager/${cert_manager_version}/cert-manager.yaml"
  version: "${cert_manager_version}"
overridesFolder: "${HOME}"
EXP_CLUSTER_RESOURCE_SET: "true"
images:
  all:
    repository: "${image_registry}/${base_project}/cluster-api"
  infrastructure-vcd:
    repository: "${image_registry}/${base_project}"
  cert-manager:
    repository: "${image_registry}/${base_project}/jetstack"
EOF

# Execute upgrade
# NOTE: clusterctl upgrade expects prefix "v" before the version to successfully execute
export KUBECONFIG=$1
${CLUSTER_PACKAGES_FOLDER}/clusterctl_artifact/clusterctl upgrade apply \
  -i capvcd-system/vcd:${capvcd_version} \
  --core cluster-api:${capi_version} \
  --bootstrap kubeadm:${kubeadm_bootstrap_version} \
  --control-plane kubeadm:${kubeadm_control_plane_version} \
  --config "${HOME}/.cluster-api/clusterctl.yaml" --v=10

# Update CPI, CSI, Projector images
kubectl set image deployment/vmware-cloud-director-ccm vmware-cloud-director-ccm="${image_registry}/${base_project}/${cpi_image_name}:${cpi_version}" -n kube-system
kubectl set image daemonset/csi-vcd-nodeplugin vcd-csi-plugin="${image_registry}/${base_project}/${csi_image_name}:${csi_version}" -n kube-system
kubectl set image deployment/rdeprojector-controller-manager manager="${image_registry}/${base_project}/${rde_projector_image_name}:${rde_projector_version}" -n rdeprojector-system
# For CSI, from 1.4.0 onwards, the controller has been changed to deployment from statefulsets. Hence, remove existing stateful set and create CSI as deployment
sed "s~__REGISTRY__~$image_registry/$base_project~g" ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-controller-crs-airgap.yaml.template > ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-controller-crs-airgap.yaml
kubectl replace --force -f ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-controller-crs-airgap.yaml
# Adding `|| true` to prevent script erroring out if Kubernetes cannot find CSI StatefulSet to delete
# This would occur if the script was ran once and had already deleted the resource, or if the CSI version being used does not have the StatefulSet
kubectl delete statefulset -n kube-system csi-vcd-controllerplugin || true

# For CPI, we will need to re-apply the updated CRS manifests as K8s v1.24+ toleration has changed to control-plane only, which previous manifests do not have.
sed "s~__REGISTRY__~$image_registry/$base_project~g" ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped/cloud-director-ccm-crs-airgap.yaml.template > ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped/cloud-director-ccm-crs-airgap.yaml
kubectl replace --force -f ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped/cloud-director-ccm-crs-airgap.yaml

echo "Upgrade of cluster-api, capvcd, cpi, csi, rde-projector components completed successfully.....!!!"

echo "Cleaning up downloaded folders from cluster upgrade script..."
remove_folder ${CLUSTER_PACKAGES_FOLDER}
remove_folder ${HOME}/.cluster-api

echo "Removed cluster packages and cluster-api folders successfully"
exit 0
